##############################################
# Script for diffusion modelling of cannabis
# use initiation.
# version of 24 January 2024
# by Christian Steglich
##############################################

# Read data sets:
load("Glasgow-friendship.RData")
load("Glasgow-demographic.RData")
load("Glasgow-substances.RData")

# encode cannabis use for diffusion modelling:
initcan <- 1*(cannabis>1)
table(initcan,cannabis,useNA='always')

# inspect inconsistent responses:
table(initcan[,1],initcan[,2])
# three cases reporting "never cannabis" after having reported sth else
table(initcan[,2],initcan[,3])
# three more cases
table(initcan[,1],initcan[,3])
# one case shows inconsistency over whole period
(inccanper1 <- which(initcan[,1]==1 & initcan[,2]==0))
(inccanper2 <- which(initcan[,2]==1 & initcan[,3]==0))
which(initcan[,1]==1 & initcan[,3]==0) # is included in former group
cannabis[inccanper1,]
cannabis[inccanper2,]
# all are cases of 2->1 response changes "only tried once -> never tried"

# RECODE cannabis initiation for inconsistent response pattern users:
initcan['s060',] <- c(1,1,1) # raw data pattern was 2  1  2
initcan['s100',] <- c(1,1,1) # raw data pattern was 2  1  2
initcan['s122',] <- c(1,1,1) # raw data pattern was 2  1  2
initcan['s014',] <- c(0,0,0) # raw data pattern was 1  2  1
initcan['s086',] <- c(1,1,1) # raw data pattern was 2  2  1
initcan['s151',] <- c(0,0,0) # raw data pattern was 1  2  1
# revisiting descriptives table above gives 10 initiations in period 1
# and another 26 initiations in period 2


# make RSiena data object:
library(RSiena)
packageVersion("RSiena") # 1.3.3
friendship.1[friendship.1==2] <- 1 # binarise!
friendship.2[friendship.2==2] <- 1 # binarise!
friendship.3[friendship.3==2] <- 1 # binarise!
thedata <- sienaDataCreate(
	friendship = sienaDependent(array(c(
		friendship.1, friendship.2, friendship.3
	), dim=c(dim(friendship.1), 3))),
	initcan = sienaDependent(initcan, type='behavior'),
	fem = coCovar(sex.F-1, centered=FALSE),
	tob = varCovar(tobacco[, 1:2]),
	alc = varCovar(alcohol[, 1:2])
)

########### CHANGING NETWORK ANALYSES ###############

# specify model:
(themodel <- getEffects(thedata))
#effectsDocumentation(themodel)
themodel <- includeEffects(themodel, totExposure, type='rate',
	name='initcan', interaction1='friendship')
themodel <- includeEffects(themodel, RateX, type='rate',
	name='initcan', interaction1='tob')
themodel <- includeEffects(themodel, RateX, type='rate',
	name='initcan', interaction1='alc')
themodel <- includeEffects(themodel, RateX, type='rate',
	name='initcan', interaction1='fem')

# new: initially set behaviour rate a bit higher to stimulate
# variability in simulated statistics
themodel <- setEffect(themodel, Rate, type='rate', period=1,
	name='initcan', initialValue=2)
themodel <- setEffect(themodel, Rate, type='rate', period=2,
	name='initcan', initialValue=2)

# new: network part! Take from "Model 3" in Table 9.4
themodel <- includeEffects(themodel, gwespFF, type='creation')
themodel <- includeEffects(themodel, gwespFF, type='endow')
themodel <- includeEffects(themodel, inPop, outAct, inAct, reciAct)
themodel <- includeEffects(themodel, egoX, altX, sameX, interaction1='fem')
themodel <- includeEffects(themodel, egoX, altX, sameX, interaction1='initcan')
themodel <- includeInteraction(themodel, gwespFF, recip)
themodel

# set algorithm controls:
theoptions1 <- sienaAlgorithmCreate(seed=1234,
	projname='cannabis-initiation',
	firstg=0.05, reduceg=0.9, # adapted; default not working
	nsub=1, n3=100) # short run, aim is just to get better initial values
theoptions2 <- sienaAlgorithmCreate(seed=1234,
	projname='cannabis-initiation',
	firstg=0.2, reduceg=0.5, # defaults
	nsub=4, n3=2500, n2start=200) # for 2nd run
theoptions3 <- sienaAlgorithmCreate(seed=1234,
	projname='cannabis-initiation',
	firstg=0.2, reduceg=0.5, # defaults
	nsub=4, n3=2500, n2start=210) # for 2nd run

# estimate:
(theresults <- siena07(theoptions1, data=thedata,
	effects=themodel)) # really bad convergence as expected
(theresults <- siena07(theoptions2, data=thedata,
	effects=themodel,prevAns=theresults)) # converged
siena.table(theresults, file='Table InitCan Network Model.htm',
	type='html', sig=TRUE)
browseURL("Table InitCan Network Model.htm")
